@@ -915,9 +915,9 @@ class RegressionVsColComponent(ExplainerComponent):
                                         "When you have some real outliers it can help to remove them"
                                         " from the plot so it is easier to see the overall pattern.", 
                                     target='reg-vs-col-winsor-label-'+self.name),
-                            dbc.Input(id='reg-vs-col-winsor-'+self.name, 
-                                    value=self.winsor,
-                                type="number", min=0, max=49, step=1),
+                                dbc.Input(id='reg-vs-col-winsor-'+self.name, 
+                                        value=self.winsor,
+                                    type="number", min=0, max=49, step=1),
                         ], md=4), hide=self.hide_winsor),  
                     make_hideable(
                         dbc.Col([
@@ -951,7 +951,10 @@ class RegressionVsColComponent(ExplainerComponent):
              Input('reg-vs-col-winsor-'+self.name, 'value')],
         )
         def update_residuals_graph(col, display, points, winsor):
-            style = {} if col in self.explainer.cats else dict(display="none")
+            if col in self.explainer.onehot_cols or col in self.explainer.categorical_cols:
+                style = {}
+            else:
+                style = dict(display="none")
             if display == 'observed':
                 return self.explainer.plot_y_vs_feature(
                         col, points=bool(points), winsor=winsor, dropna=True), style
